@@ -1,16 +1,19 @@
+# -*- coding: utf-8 -*-
 """GNUmed authentication widgets.
 
 This module contains widgets and GUI
 functions for authenticating users.
 """
 #================================================================
-__version__ = "$Revision: 1.45 $"
 __author__ = "karsten.hilbert@gmx.net, H.Herb, H.Berger, R.Terry"
 __license__ = "GPL v2 or later (details at http://www.gnu.org)"
 
 
 # stdlib
-import sys, os.path, logging, re as regex
+import sys
+import os.path
+import logging
+import re as regex
 
 
 # 3rd party
@@ -20,13 +23,21 @@ import wx
 # GNUmed
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
-from Gnumed.pycommon import gmLoginInfo, gmPG2, gmBackendListener, gmTools, gmCfg2, gmI18N
-from Gnumed.business import gmSurgery
-from Gnumed.wxpython import gmGuiHelpers, gmExceptionHandlingWidgets
+from Gnumed.pycommon import gmLoginInfo
+from Gnumed.pycommon import gmPG2
+from Gnumed.pycommon import gmBackendListener
+from Gnumed.pycommon import gmTools
+from Gnumed.pycommon import gmCfg2
+from Gnumed.pycommon import gmI18N
+from Gnumed.pycommon import gmLog2
+
+from Gnumed.business import gmPraxis
+
+from Gnumed.wxpython import gmGuiHelpers
+from Gnumed.wxpython import gmExceptionHandlingWidgets
 
 
 _log = logging.getLogger('gm.ui')
-_log.info(__version__)
 _cfg = gmCfg2.gmCfgData()
 
 try:
@@ -125,6 +136,8 @@ def connect_to_database(max_attempts=3, expected_version=None, require_version=T
 			_log.info("user cancelled login dialog")
 			break
 
+		gmLog2.add_word2hide(login.password)
+
 		# try getting a connection to verify the DSN works
 		dsn = gmPG2.make_psycopg2_dsn (
 			database = login.database,
@@ -149,6 +162,10 @@ def connect_to_database(max_attempts=3, expected_version=None, require_version=T
 						'\n'
 						"Please retry with proper credentials or cancel.\n"
 						'\n'
+						' (for the public and any new GNUmed data-\n'
+						'  bases the default user name and password\n'
+						'  are {any-doc, any-doc})\n'
+						'\n'
 						'You may also need to check the PostgreSQL client\n'
 						'authentication configuration in pg_hba.conf. For\n'
 						'details see:\n'
@@ -161,6 +178,9 @@ def connect_to_database(max_attempts=3, expected_version=None, require_version=T
 						"%s\n\n"
 						"Please retry with proper credentials or cancel.\n"
 						"\n"
+						"For the public and any new GNUmed databases the\n"
+						"default user name and password are {any-doc, any-doc}.\n"
+						"\n"
 						'You may also need to check the PostgreSQL client\n'
 						'authentication configuration in pg_hba.conf. For\n'
 						'details see:\n'
@@ -176,19 +196,26 @@ def connect_to_database(max_attempts=3, expected_version=None, require_version=T
 			del e
 			continue
 
-		except gmPG2.dbapi.OperationalError, e:
-			_log.error(u"login attempt failed: %s", e)
+		except gmPG2.dbapi.OperationalError, exc:
+			exc = gmPG2.make_pg_exception_fields_unicode(exc)
+			_log.error(u"login attempt failed: %s", exc)
 			msg = _(
 				"Unable to connect to database:\n\n"
 				"%s\n\n"
 				"Please retry another backend / user / password combination !\n"
-			) % gmPG2.extract_msg_from_pg_exception(e)
+				"\n"
+				" (for the public and any new GNUmed databases\n"
+				"  the default user name and password are\n"
+				"  {any-doc, any-doc})\n"
+				"\n"
+			) % exc.u_pgerror
+#			) % gmPG2.extract_msg_from_pg_exception(e)
 			msg = regex.sub(r'password=[^\s]+', u'password=%s' % gmTools.u_replacement_character, msg)
 			gmGuiHelpers.gm_show_error (
 				msg,
 				_('Connecting to backend')
 			)
-			del e
+			del exc
 			continue
 
 		# connect was successful
@@ -263,6 +290,7 @@ def connect_to_database(max_attempts=3, expected_version=None, require_version=T
 	dlg.Destroy()
 
 	return connected
+
 #================================================================
 def get_dbowner_connection(procedure=None, dbo_password=None, dbo_account=u'gm-dbo'):
 	if procedure is None:
@@ -286,6 +314,8 @@ Please enter the current password for <%s>:""") % (
 		if dbo_password == '':
 			return None
 
+	gmLog2.add_word2hide(dbo_password)
+
 	# 2) connect as gm-dbo
 	login = gmPG2.get_default_login()
 	dsn = gmPG2.make_psycopg2_dsn (
@@ -312,6 +342,7 @@ Please enter the current password for <%s>:""") % (
 		return None
 
 	return conn
+
 #================================================================
 def change_gmdbowner_password():
 
@@ -340,6 +371,8 @@ def change_gmdbowner_password():
 	if dbo_pwd_new_1.strip() == u'':
 		return False
 
+	gmLog2.add_word2hide(dbo_pwd_new_1)
+
 	dbo_pwd_new_2 = wx.GetPasswordFromUser (
 		message = _(u"""Enter the NEW password for the GNUmed database owner, again.
 
@@ -353,6 +386,49 @@ def change_gmdbowner_password():
 	if dbo_pwd_new_1 != dbo_pwd_new_2:
 		return False
 
+	# pwd2 == pwd1 at this point so no need to hide (again)
+
+	"""	On Mon, Mar 13, 2017 at 12:19:22PM -0400, Tom Lane wrote:
+		> Date: Mon, 13 Mar 2017 12:19:22 -0400
+		> From: Tom Lane <tgl@sss.pgh.pa.us>
+		> To: Adrian Klaver <adrian.klaver@aklaver.com>
+		> cc: Schmid Andreas <Andreas.Schmid@bd.so.ch>,
+		>  "'pgsql-general@postgresql.org'" <pgsql-general@postgresql.org>
+		> Subject: Re: [GENERAL] createuser: How to specify a database to connect to
+		> 
+		> Adrian Klaver <adrian.klaver@aklaver.com> writes:
+		> > On 03/13/2017 08:52 AM, Tom Lane wrote:
+		> >> If by "history" you're worried about the server-side statement log, this
+		> >> is merest fantasy: the createuser program is not magic, it just constructs
+		> >> and sends a CREATE USER command for you.  You'd actually be more secure
+		> >> using psql, where (if you're superuser) you could shut off log_statement
+		> >> for your session first.
+		> 
+		> > There is a difference though:
+		> 
+		> > psql> CREATE USER:
+		> 
+		> > postgres-2017-03-13 09:03:27.147 PDT-0LOG:  statement: create user 
+		> > dummy_user with login password '1234';
+		> 
+		> Well, what you're supposed to do is
+		> 
+		> postgres=# create user dummy_user;
+		> postgres=# \password dummy_user
+		> Enter new password: 
+		> Enter it again: 
+		> postgres=# 
+		> 
+		> which will result in sending something like
+		> 
+		> ALTER USER dummy_user PASSWORD 'md5c5e9567bc40082671d02c654260e0e09'
+		> 
+		> You can additionally protect that by wrapping it into one transaction
+		> (if you have a setup where the momentary existence of the role without a
+		> password would be problematic) and/or shutting off logging beforehand.
+	"""
+
+	# this REALLY should be prefixed with md5 and the md5sum sent rather than the pwd
 	cmd = u"""ALTER ROLE "%s" ENCRYPTED PASSWORD '%s';""" % (
 		dbo_account,
 		dbo_pwd_new_2
@@ -360,20 +436,23 @@ def change_gmdbowner_password():
 	gmPG2.run_rw_queries(link_obj = dbo_conn, queries = [{'cmd': cmd}], end_tx = True)
 
 	return True
+
 #================================================================
 class cBackendProfile:
 	pass
+
 #================================================================
 class cLoginDialog(wx.Dialog):
 	"""cLoginDialog - window holding cLoginPanel"""
 
-	def __init__(self, parent, id, title=_("Welcome to the"), client_version=u'*** unknown ***'):
+	def __init__(self, parent, id, title = _("Welcome to"), client_version = u'*** unknown ***'):
 		wx.Dialog.__init__(self, parent, id, title)
 		self.panel = cLoginPanel(self, -1, isDialog=1, client_version = client_version)
 		self.Fit() # needed for Windoze.
 		self.Centre()
 
 		self.SetIcon(gmTools.get_icon(wx = wx))
+
 #================================================================
 class cLoginPanel(wx.Panel):
 	"""GUI panel class that interactively gets Postgres login parameters.
@@ -411,7 +490,7 @@ class cLoginPanel(wx.Panel):
 			self.topsizer.Add (
 				bmp,
 				proportion = 0,
-				flag = wx.GROW|wx.ALIGN_CENTER_VERTICAL|wx.ALL,
+				flag = wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_CENTER_HORIZONTAL|wx.ALL,
 				border = 10
 			)
 		except:
@@ -427,7 +506,7 @@ class cLoginPanel(wx.Panel):
 				border = 10
 			)
 
-		paramsbox_caption = _('"%s" (version %s)') % (gmSurgery.gmCurrentPractice().active_workplace, client_version)
+		paramsbox_caption = _('Workplace "%s" (version %s)') % (gmPraxis.gmCurrentPraxisBranch().active_workplace, client_version)
 
 		# FIXME: why doesn't this align in the centre ?
 		self.paramsbox = wx.StaticBox( self, -1, paramsbox_caption, style = wx.ALIGN_CENTRE_HORIZONTAL)
@@ -453,7 +532,7 @@ class cLoginPanel(wx.Panel):
 			-1,
 			self.__backend_profiles.keys()[0],
 			wx.DefaultPosition,
-			size = wx.Size(150,-1),
+			size = wx.Size(550,-1),
 			choices = self.__backend_profiles.keys(),
 			style = wx.CB_READONLY
 		)
@@ -707,56 +786,67 @@ class cLoginPanel(wx.Panel):
 	#############################################################################
 	def GetLoginInfo(self):
 		"""convenience function for compatibility with gmLoginInfo.LoginInfo"""
-		if not self.cancelled:
-			# FIXME: do not assume conf file is latin1 !
-			#profile = self.__backend_profiles[self._CBOX_profile.GetValue().encode('latin1').strip()]
-			profile = self.__backend_profiles[self._CBOX_profile.GetValue().encode('utf8').strip()]
-			_log.debug(u'backend profile "%s" selected', profile.name)
-			_log.debug(u' details: <%s> on %s@%s:%s (%s, %s)',
-				self._CBOX_user.GetValue(),
-				profile.database,
-				profile.host,
-				profile.port,
-				profile.encoding,
-				gmTools.bool2subst(profile.public_db, u'public', u'private')
-			)
-			_log.debug(u' helpdesk: "%s"', profile.helpdesk)
-			login = gmLoginInfo.LoginInfo (
-				user = self._CBOX_user.GetValue(),
-				password = self.pwdentry.GetValue(),
-				host = profile.host,
-				database = profile.database,
-				port = profile.port
-			)
-			login.public_db = profile.public_db
-			login.helpdesk = profile.helpdesk
-			return login
+		if self.cancelled:
+			return None
 
-		return None
+		# FIXME: do not assume conf file is latin1 !
+		#profile = self.__backend_profiles[self._CBOX_profile.GetValue().encode('latin1').strip()]
+		profile = self.__backend_profiles[self._CBOX_profile.GetValue().encode('utf8').strip()]
+		_log.info(u'backend profile "%s" selected', profile.name)
+		_log.info(u' details: <%s> on %s@%s:%s (%s, %s)',
+			self._CBOX_user.GetValue(),
+			profile.database,
+			profile.host,
+			profile.port,
+			profile.encoding,
+			gmTools.bool2subst(profile.public_db, u'public', u'private')
+		)
+		_log.info(u' helpdesk: "%s"', profile.helpdesk)
+		login = gmLoginInfo.LoginInfo (
+			user = self._CBOX_user.GetValue(),
+			password = self.pwdentry.GetValue(),
+			host = profile.host,
+			database = profile.database,
+			port = profile.port
+		)
+		login.public_db = profile.public_db
+		login.helpdesk = profile.helpdesk
+		login.backend_profile = profile.name
+		return login
 	#----------------------------
 	# event handlers
 	#----------------------------
 	def OnHelp(self, event):
-		praxis = gmSurgery.gmCurrentPractice()
+		praxis = gmPraxis.gmCurrentPraxisBranch()
 		wx.MessageBox(_(
-"""GNUmed main login screen
+u"""Unable to connect to the database ?
 
-USER:
- name of the GNUmed user
-PASSWORD
- password for this user
+ "PostgreSQL: FATAL:  password authentication failed ..."
 
-button OK:
- proceed with login
-button OPTIONS:
- set advanced options
-button CANCEL:
- abort login and quit GNUmed client
-button HELP:
- this help screen
+The default user name and password are {any-doc, any-doc}
+for the public and any new GNUmed databases.
 
-For assistance on using GNUmed please contact:
- %s""") % praxis.helpdesk)
+ "... could not connect to server ..."
+
+Mostly this is a case of new users who did not yet install
+or configure a PostgreSQL server and/or a GNUmed database
+of their own, which you must do before you can connect to
+anything other than the public demonstration database, see
+
+ http://wiki.gnumed.de/bin/view/Gnumed/GmManualServerInstall
+
+For assistance on using GNUmed please consult the wiki:
+
+ http://wiki.gnumed.de/bin/view/Gnumed/GnumedManual
+
+For more help than the above, please contact:
+
+ GNUmed Development List <gnumed-bugs@gnu.org>
+
+For local assistance please contact:
+
+ %s""") % praxis.helpdesk,
+ 		caption = _('HELP for GNUmed main login screen'))
 
 	#----------------------------
 	def __on_login_button_pressed(self, event):
@@ -830,4 +920,3 @@ if __name__ == "__main__":
 #		app.MainLoop()
 
 #================================================================
-
